/*
    Copyright (C) 2008  Colin Caprani - www.colincaprani.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// main file for the Runge-Kutta program

#include <iostream>
#include <stdlib.h>
#include "Math.h"

using namespace std;

double rk4(double x0, double y0, double h, int N, double (*pf)(double, double));
double rkn(double x0, double y0, double dydx0, double h, int N, double (*pf)(double, double, double));
double f1(double x, double y);
double f2(double x, double y);
double f3(double x, double y, double dydx);

void main()
{
	double ans1 = 0.0;
	double ans2 = 0.0;
	double (*pFrk4)(double, double);
	double (*pFrkn)(double, double, double);
	
	pFrk4 = f2;
	ans1 = rk4(0,1,0.2,5, pFrk4);

	pFrkn = f3;
	ans2 = rkn(0,0,0,0.2,5, pFrkn);

	cout << ans1 <<'\t' << ans2 << endl;
}

double f1(double x, double y)
{
	return x*y;
}

double f2(double x, double y)
{
	return pow((y - x - 1),2) + 2;
}

double f3(double x, double y, double dydx)
{
	return 0.5 * (x + y + dydx + 2);
}

double rkn(double x0, double y0, double dydx0, double h, int N, double (*pf)(double, double, double))
{
	double k1, k2, k3, k4;
	k1 = 0.0; k2 = 0.0; k3 = 0.0; k4 = 0.0;
	double xi, yi, dydxi;
	xi = 0.0; yi = 0.0; dydxi = 0.0;
	double K, L;
	K = 0.0, L = 0.0;
	
	xi = x0; yi = y0; dydxi = dydx0;

	for(int i = 0; i <= N - 1; i++)
	{
		k1 = 0.5 * h * pf(xi, yi, dydxi);
		
		K = 0.5 * h * (dydxi + 0.5 * k1);
		k2 = 0.5 * h * pf(xi + 0.5*h, yi + K, dydxi + k1);
		
		k3 = 0.5 * h * pf(xi + 0.5*h, yi + K, dydxi + k2);
		
		L = h * (dydxi + k3);
		k4 = 0.5 * h * pf(xi + h, yi + L, dydxi + 2*k3);
		xi += h;
		
		yi = yi + h * (dydxi + (k1 + k2 + k3)/3);
		dydxi = dydxi + (k1 + 2*k2 + 2*k3 + k4)/3;
	}

	return yi;
}

double rk4(double x0, double y0, double h, int N, double (*pf)(double, double))
{
	double k1, k2, k3, k4;
	k1 = 0.0; k2 = 0.0; k3 = 0.0; k4 = 0.0;
	double xi, yi;
	xi = 0.0; yi = 0.0;
	
	xi = x0; yi = y0;

	for(int i = 0; i <= N - 1; i++)
	{
		k1 = h * pf(xi, yi);
		k2 = h * pf(xi + 0.5 * h, yi + 0.5 * k1);
		k3 = h * pf(xi + 0.5 * h, yi + 0.5 * k2);
		k4 = h * pf(xi + h, yi + k3);
		xi += h;
		yi = yi + (k1 + 2*k2 + 2*k3 + k4)/6;
	}

	return yi;
}